import unittest
import responses
from datetime import datetime
from gls import GLS, Address, Parcel


ADDRESS = {
    "City": "Budapest",
    "ContactEmail": "info@bme.hu",
    "ContactName": "Dr. Sztoczek József",
    "ContactPhone": "+36 1 463-1111",
    "CountryIsoCode": "HU",
    "HouseNumber": "3",
    "Name": "Budapesti Műszaki és Gazdaságtudományi Egyetem",
    "Street": "Műegyetem rkp.",
    "ZipCode": "1111",
    "HouseNumberInfo": "",
}


class TestGLS(unittest.TestCase):
    def test_create_parcel(self):
        gls = GLS(123, "", "")

        parcel: Parcel = gls.create_parcel(
            pickup_from=Address(**ADDRESS),
            deliver_to=Address(**ADDRESS),
            pickup_date=datetime(2022, 4, 10, 12, 30),
            reference="REF-123",
            count=1,
            content="paper",
            cod_reference="cod-321",
            cod_amount=456,
        )

        self.assertEqual(123, parcel.ClientNumber)
        self.assertEqual("REF-123", parcel.ClientReference)
        self.assertEqual("1649608200000", parcel.PickupDate)
        self.assertEqual("Budapest", parcel.PickupAddress.City)
        self.assertEqual("Budapest", parcel.DeliveryAddress.City)
        self.assertEqual(1, parcel.Count)
        self.assertEqual("paper", parcel.Content)
        self.assertEqual("cod-321", parcel.CODReference)
        self.assertEqual(456, parcel.CODAmount)
        self.assertEqual([], parcel.ServiceList)

    @responses.activate
    def test_get_parcels(self):

        responses.add(
            responses.POST,
            "https://api.mygls.hu/ParcelService.svc/json/GetParcelList",
            json={"GetParcelListErrors": [], "PrintDataInfoList": []},
            status=200,
        )

        gls = GLS(123, "", "")
        result = gls.get_parcels(
            print_from=datetime(2021, 11, 1, 12, 30),
            print_to=datetime(2021, 11, 24, 12, 30),
        )
        self.assertEqual({"GetParcelListErrors": [], "PrintDataInfoList": []}, result)

    @responses.activate
    def test_get_printed_labels(self):
        responses.add(
            responses.POST,
            "https://api.mygls.hu/ParcelService.svc/json/GetPrintedLabels",
            json={"GetPrintedLabelsErrorList": [], "Labels": [10, 11, 22, 23]},
            status=200,
        )
        gls = GLS(123, "", "")
        result = gls.get_printed_labels(parcel_ids=[1, 2])
        self.assertEqual(
            {"GetPrintedLabelsErrorList": [], "Labels": [10, 11, 22, 23]}, result
        )

    @responses.activate
    def test_delete_labels(self):
        responses.add(
            responses.POST,
            "https://api.mygls.hu/ParcelService.svc/json/DeleteLabels",
            json={
                "SuccessfullyDeletedList": [{"ParcelId": 1, "SubParcelIdList": []}],
                "DeleteLabelsErrorList": [],
            },
            status=200,
        )
        gls = GLS(123, "", "")
        result = gls.delete_labels(parcel_ids=[1])
        self.assertEqual(1, result.SuccessfullyDeletedList[0].ParcelId)

    @responses.activate
    def test_prepare_labels(self):
        responses.add(
            responses.POST,
            "https://api.mygls.hu/ParcelService.svc/json/PrepareLabels",
            json={
                "ParcelInfoList": [],
                "PrepareLabelsError": [],
            },
            status=200,
        )
        gls = GLS(123, "", "")

        parcel: Parcel = gls.create_parcel(
            pickup_from=Address(**ADDRESS),
            deliver_to=Address(**ADDRESS),
            pickup_date=datetime(2022, 4, 10, 12, 30),
            reference="REF-123",
            count=1,
        )

        result = gls.prepare_labels([parcel])
        self.assertEqual(
            {
                "ParcelInfoList": [],
                "PrepareLabelsError": [],
            },
            result,
        )



from enum import Enum


class Code(Enum):
    H24 = "24H"
    ADR = "ADR"
    AOS = "AOS"
    COD = "COD"
    CS1 = "CS1"
    DDS = "DDS"
    DPV = "DPV"
    FDS = "FDS"
    FSS = "FSS"
    INS = "INS"
    PRS = "PRS"
    PSD = "PSD"
    PSS = "PSS"
    SAT = "SAT"
    SBS = "SBS"
    SDS = "SDS"
    SM1 = "SM1"
    SM2 = "SM2"
    SRS = "SRS"
    SZL = "SZL"
    T09 = "T09"
    T10 = "T10"
    T12 = "T12"
    TGS = "TGS"
    XS = "XS"


class PrinterType(Enum):
    A4_2x2 = "A4_2x2"
    A4_4x1 = "A4_4x1"
    CONNECT = "Connect"
    THERMO = "Thermo"



from enum import Enum
from pydantic import BaseSettings


class CountryCode(Enum):
    HU = "hu"
    HR = "hr"
    CZ = "cz"
    RO = "ro"
    SI = "si"
    SK = "sk"


class Settings(BaseSettings):
    country: CountryCode = CountryCode.HU
    test: bool = False
    timeout_seconds: int = 5

    @property
    def api_root(self):
        test = ".test" if self.test else ""
        return f"https://api{test}.mygls.{self.country.value}/ParcelService.svc/json"



"""
Parcel class
"""

from pydantic.dataclasses import dataclass
from typing import Optional
from .address import Address
from .service import Service


@dataclass
class Parcel:
    ClientNumber: int
    ClientReference: Optional[str]
    Count: int
    DeliveryAddress: Address
    PickupAddress: Address
    PickupDate: str
    ServiceList: list[Service]
    Content: Optional[str] = ""
    CODAmount: Optional[float] = 0
    CODReference: Optional[str] = ""



from pydantic.dataclasses import dataclass
from typing import Optional
from .gls_types import Code


@dataclass
class ServiceParameterString:
    Value: Optional[str]


@dataclass
class ServiceParameterDateTime:
    Value: Optional[str]


@dataclass
class ServiceParameterADR:
    AdrItemType: int
    AmountUnit: int
    InnerCount: int
    PackSize: int
    UnNumber: int


@dataclass
class ServiceParameterStringDecimal:
    StringValue: str
    DecimalValue: float


@dataclass
class ServiceParameterDecimal:
    Value: Optional[float]


@dataclass
class ServiceParameterTimeRange:
    TimeFrom: str
    TimeTo: str


@dataclass
class PSDParameter:
    StringValue: str


@dataclass
class Service:
    Code: Code
    ADRParameter: Optional[ServiceParameterADR]
    AOSParameter: Optional[ServiceParameterString]
    CS1Parameter: Optional[ServiceParameterString]
    DDSParameter: Optional[ServiceParameterDateTime]
    DPVParameter: Optional[ServiceParameterStringDecimal]
    FDSParameter: Optional[ServiceParameterString]
    FSSParameter: Optional[ServiceParameterString]
    INSParameter: Optional[ServiceParameterDecimal]
    MMPParameter: Optional[ServiceParameterDecimal]
    PSDParameter: Optional[PSDParameter]
    SDSParameter: Optional[ServiceParameterTimeRange]
    SM1Parameter: Optional[ServiceParameterString]
    SM2Parameter: Optional[ServiceParameterString]
    SZLParameter: Optional[ServiceParameterString]
    Value: Optional[str]



import hashlib
import json
import logging
import requests
from datetime import datetime
from typing import Optional
from dataclasses import asdict
from .parcel import Parcel
from .response import (
    ParcelResponse,
    PrintedLabelsResponse,
    PrepareLabelsResponse,
    DeleteLabelsResponse,
)
from .address import Address
from .gls_types import PrinterType
from .service import Service
from .settings import Settings

HEADERS = {"Content-type": "application/json"}

log = logging.getLogger(__name__)


class GLS:
    """
    GLS API client

    https://api.mygls.hu/index_en.html
    """

    def __init__(
        self,
        client_number: int,
        username: str,
        password: str,
        settings: Settings = Settings(),
    ):
        """
        Args: client_number: MyGLS client number
              username: MyGLS username
              password: MyGLS password
              timeout_seconds: network request timeout in seconds
        """
        self.username = username
        self.client_number = client_number
        self.password = self._calculate_password(password)
        self.settings = settings

    def convert_to_datefield(self, date: datetime) -> str:
        """
        Converts date object to API format: /Date(timestamp)/
        """
        return f"/Date({self._convert_to_timestamp(date)})/"

    def create_parcel(
        self,
        pickup_from: Address,
        deliver_to: Address,
        pickup_date: datetime,
        reference: str,
        count: int,
        content: Optional[str] = "",
        cod_amount: Optional[float] = 0,
        cod_reference: Optional[str] = "",
        services: Optional[list[Service]] = [],
    ) -> Parcel:
        """
        Creates a parcel object
        """
        return Parcel(
            ClientNumber=self.client_number,
            ClientReference=reference,
            Content=content,
            PickupAddress=pickup_from,
            DeliveryAddress=deliver_to,
            PickupDate=self._convert_to_timestamp(pickup_date),
            Count=count,
            CODAmount=cod_amount,
            CODReference=cod_reference,
            ServiceList=services,
        )

    def get_parcels(
        self,
        pickup_from: Optional[datetime] = None,
        pickup_to: Optional[datetime] = None,
        print_from: Optional[datetime] = None,
        print_to: Optional[datetime] = None,
    ) -> ParcelResponse:
        """
        Gets parcels optionally filtered by pickup and print data ranges
        """
        payload = self._request_payload()
        if pickup_from:
            payload["PickupDateFrom"] = self.convert_to_datefield(pickup_from)
        if pickup_to:
            payload["PickupDateTo"] = self.convert_to_datefield(pickup_to)
        if print_from:
            payload["PrintDateFrom"] = self.convert_to_datefield(print_from)
        if print_to:
            payload["PrintDateTo"] = self.convert_to_datefield(print_to)
        response = requests.post(
            f"{self.settings.api_root}/GetParcelList",
            data=json.dumps(payload),
            headers=HEADERS,
            timeout=self.settings.timeout_seconds,
        )
        return ParcelResponse.__pydantic_model__.parse_raw(response.text)

    def get_printed_labels(
        self,
        parcel_ids: list[int],
        printer_type: PrinterType = PrinterType.THERMO,
        print_position: int = 1,
        show_dialog: bool = False,
    ) -> PrintedLabelsResponse:
        """
        Gets printed labels
        """
        payload = self._request_payload()
        payload["ParcelIdList"] = parcel_ids
        payload["PrintPosition"] = print_position
        payload["ShowPrintDialog"] = 1 if show_dialog else 0
        payload["TypeOfPrinter"] = printer_type.value
        response = requests.post(
            f"{self.settings.api_root}/GetPrintedLabels",
            data=json.dumps(payload),
            headers=HEADERS,
            timeout=self.settings.timeout_seconds,
        )
        return PrintedLabelsResponse.__pydantic_model__.parse_raw(response.text)

    def print_labels(
        self,
        pdf_path: str,
        parcels: list[Parcel],
        printer_type: PrinterType = PrinterType.THERMO,
    ) -> list[int]:
        """
        Creates a parcel label and saves it as a PDF on the designated path
        """
        label_info = self.prepare_labels(parcels)
        parcel_ids = [label.ParcelId for label in label_info.ParcelInfoList]
        log.info(f"Prepared labels: {parcel_ids}")
        labels = self.get_printed_labels(parcel_ids, printer_type)
        label_data = labels.Labels
        self._save_pdf(pdf_path, label_data)
        return parcel_ids

    def prepare_labels(self, parcels: list[Parcel]) -> PrepareLabelsResponse:
        """
        Prepares labels for printing
        """
        payload = self._request_payload()
        payload["ParcelList"] = [asdict(p) for p in parcels]
        response = requests.post(
            f"{self.settings.api_root}/PrepareLabels",
            data=json.dumps(payload),
            headers=HEADERS,
            timeout=self.settings.timeout_seconds,
        )
        return PrepareLabelsResponse.__pydantic_model__.parse_raw(response.text)

    def delete_labels(self, parcel_ids: list[int]) -> DeleteLabelsResponse:
        """
        Deletes labels
        """
        payload = self._request_payload()
        payload["ParcelIdList"] = parcel_ids
        response = requests.post(
            f"{self.settings.api_root}/DeleteLabels",
            data=json.dumps(payload),
            headers=HEADERS,
            timeout=self.settings.timeout_seconds,
        )
        return DeleteLabelsResponse.__pydantic_model__.parse_raw(response.text)

    def _calculate_password(self, plaintext: str) -> list[int]:
        """
        Calculates password for API authentication
        """
        sha = hashlib.sha512()
        sha.update(plaintext.encode("utf-8"))
        return list(sha.digest())

    def _convert_to_timestamp(self, date: datetime) -> int:
        """
        Converts date object to timestamp
        """
        return int(datetime.timestamp(date)) * 1000

    def _request_payload(self) -> dict:
        return {"Username": self.username, "Password": self.password}

    def _save_pdf(self, pdf_path: str, byte_list: list[int]) -> None:
        data = bytes(byte_list)
        with open(pdf_path, "wb") as f:
            f.write(data)
        log.info(f"Saved parcel label to {pdf_path}")



from pydantic.dataclasses import dataclass
from typing import Optional


@dataclass
class Address:
    City: str
    ContactEmail: Optional[str]
    ContactName: Optional[str]
    ContactPhone: Optional[str]
    CountryIsoCode: str
    HouseNumber: str
    Name: str
    Street: str
    ZipCode: str
    HouseNumberInfo: Optional[str]



import logging
from logging import NullHandler

from .gls import GLS
from .address import Address
from .gls_types import Code, PrinterType
from .parcel import Parcel
from .response import (
    ErrorInfo,
    ParcelInfo,
    ParcelResponse,
    PrintDataInfo,
    PrintedLabelsResponse,
    PrepareLabelsResponse,
    DeletedParcelInfo,
    DeleteLabelsResponse,
)
from .service import (
    ServiceParameterString,
    ServiceParameterDateTime,
    ServiceParameterADR,
    ServiceParameterStringDecimal,
    ServiceParameterDecimal,
    ServiceParameterTimeRange,
    PSDParameter,
    Service,
)
from .settings import CountryCode, Settings

logging.getLogger(__name__).addHandler(NullHandler())

__all__ = ["gls", "address", "gls_types", "parcel", "response", "service", "settings"]



from pydantic.dataclasses import dataclass
from typing import Optional
from .parcel import Parcel


@dataclass
class ErrorInfo:
    ErrorCode: int
    ErrorDescription: str
    ClientReferenceList: list[str]
    ParcelIdList: list[int]


@dataclass
class PrintDataInfo:
    Parcel: Parcel
    ParcelId: int
    ParcelNumber: float
    ParcelNumberWithCheckdigit: float
    DepotNumber: Optional[str]
    Driver: Optional[str]
    Depot: Optional[str]
    Sort: Optional[str]
    B2CChar: Optional[str]
    ClientReference: Optional[str]


@dataclass
class ParcelInfo:
    ClientReference: str
    ParcelId: int


@dataclass
class ParcelResponse:
    GetParcelListErrors: list[ErrorInfo]
    PrintDataInfoList: list[PrintDataInfo]


@dataclass
class PrintedLabelsResponse:
    Labels: Optional[list[int]]
    GetPrintedLabelsErrorList: list[ErrorInfo]


@dataclass
class PrepareLabelsResponse:
    ParcelInfoList: list[ParcelInfo]
    PrepareLabelsError: list[ErrorInfo]


@dataclass
class DeletedParcelInfo:
    ParcelId: int
    SubParcelIdList: list[int]


@dataclass
class DeleteLabelsResponse:
    SuccessfullyDeletedList: list[DeletedParcelInfo]
    DeleteLabelsErrorList: list[ErrorInfo]
